package com.example.afrishop_v3.controllers;

import com.example.afrishop_v3.base.Result;
import com.example.afrishop_v3.bo.KeyConstant;
import com.example.afrishop_v3.enums.ResultCodeEnum;
import com.example.afrishop_v3.models.TbCfUserInfo;
import com.example.afrishop_v3.repository.UserRepository;
import com.example.afrishop_v3.security.services.AuthenticationUser;
import com.example.afrishop_v3.util.SMSUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.Optional;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@RestController
@RequestMapping("user")
public class UserController extends Controller {
    private final UserRepository repository;
    private final AuthenticationUser user;
    private static Logger logger = LoggerFactory.getLogger(UserController.class);

    public UserController(UserRepository repository, AuthenticationUser user) {
        this.repository = repository;
        this.user = user;
    }

    private static boolean isEmailValid(String email) {
        String expression = "^[\\w.-]+@([\\w\\-]+\\.)+[A-Z]{2,4}$";
        Pattern pattern = Pattern.compile(expression, Pattern.CASE_INSENSITIVE);
        Matcher matcher = pattern.matcher(email);
        return matcher.matches();
    }

    @PutMapping("bindPhoneOrEmail")
    public Result bindPhoneOrEmail(@RequestParam(value = "email",required = false) String email,@RequestParam(value = "code",required = false) String code,@RequestParam(value = "phone",required = false) String phone){
        TbCfUserInfo user = this.user.user();

        if( StringUtils.isNotBlank(email) ){

            if( isEmailValid(email) ) user.setEmail(email);

            else return new Result(ResultCodeEnum.ILLEGAL_ARGUMENT.getCode(),"Email address is not valid");

        }


        if( StringUtils.isNotBlank(phone) ){

            if( StringUtils.isNotBlank(code) && code.equals(user.getVerificationCode()) ){
                user.setPhone(phone);
                user.setVerificationCode(null);
            }else return new Result(ResultCodeEnum.VALIDATE_ERROR.getCode(),"Code verification error");

        }

        TbCfUserInfo save = repository.save(user);

        return new Result<>(save);
    }



    @GetMapping("/identifyCode/{phone}")
    public Result getUserPhoneCode(@PathVariable("phone") String phone) {
        TbCfUserInfo user = this.user.user();

        Result result = new Result();
        try {
            //手机号拼接"+"
            phone = "+" + phone.trim();
            /**
             * 验证码时效5分钟，如果用户连续发送3次请求，则提示用户：操作频繁，请15分钟后重试
             */
            //先从缓存中获取用户的发送次数
            int sentCount = user.getSentCount();

            System.err.println(sentCount);
            //用户请求次数小于3次，则正常发送验证码，并且有效期为5分钟

            Date date = new Date();

            Date sentTime = user.getCodeSentTime();


            long diff = date.getTime() - (sentTime == null ? 0 : sentTime.getTime());

            long diffMinutes = diff / (60 * 1000) % 60;

            if( diffMinutes > 15 && sentCount >= 3 ){
                sentCount = -1;
                user.setSentCount(sentCount);
            }

            boolean canSend = sentCount < 3;


            if ( canSend ) {
                result.setMessage("Verification code has been sent");
                //发送验证码
                String code = SMSUtil.yzCode(phone);
                user.setCodeSentTime(date);
                user.setVerificationCode(code);
                user.setSentCount(user.getSentCount()+1);
                repository.save(user);
                logger.info("用户[" + phone + "]获取验证码成功，" + "验证码：" + code);
            } else {
                //用户频繁操作
                result.setCode(ResultCodeEnum.SERVICE_ERROR.getCode());
                result.setMessage("Frequent operation, please try again after 15 minutes");
                logger.info("用户[" + phone + "]获取验证码失败，操作频繁，15分钟后重试");
            }
        } catch (Exception e) {
            result.setCode(ResultCodeEnum.SERVICE_ERROR.getCode());
            result.setMessage(e.toString());
            logger.info("用户[" + phone + "]获取验证码发生异常--->>>" + e.toString());
        }
        return result;
    }

    @PutMapping
    public Result updateUser(@RequestBody TbCfUserInfo info) {
        TbCfUserInfo user = this.user.user();

        if (info == null) return new Result(ResultCodeEnum.VALIDATE_ERROR.getCode(), "Empty body");

        if(StringUtils.isNotBlank(info.getAvatar()) ){
            user.setAvatar(info.getAvatar());
        }

        if(StringUtils.isNotBlank(info.getBirthday()) ){
            user.setBirthday(info.getBirthday());
        }

        if(StringUtils.isNotBlank(info.getNick()) ){
            user.setNick(info.getNick());
        }

        if( info.getSex() != null ){
            user.setSex(info.getSex());
        }


        TbCfUserInfo save = repository.save(user);


        return new Result<>(save);

    }

    @GetMapping("/userById/{id}")
    public Result userById(@PathVariable("id") String id){
        Optional<TbCfUserInfo> byId = repository.findById(id);
        return byId.map(userInfo -> new Result<>(userInfo, "Success")).orElseGet(() -> new Result<>(ResultCodeEnum.VALIDATE_ERROR.getCode(), "Not found !"));

    }

    @GetMapping("/sendNotification/{id}")
    public Result sendFcm(@PathVariable("id") String id,@RequestParam("content") String content,@RequestParam("title") String title){
        Optional<TbCfUserInfo> byId = repository.findById(id);

        if( byId.isPresent() && byId.get().hasFcm() ){
            TbCfUserInfo userInfo = byId.get();
            sendNotification(userInfo.getFcm(),title,content);
            return new Result();
        }

        return new Result(ResultCodeEnum.SUCCESS.getCode(),"No firebase messaging token found");
    }


    @GetMapping(value = "/userByCode/{code}")
    public Result getUserByCode(@PathVariable("code") String code) {
        Optional<TbCfUserInfo> byCode = repository.findByCode(code);
        boolean present = byCode.isPresent();
        return new Result<>(present ? byCode.get() : null,present ? 1 : 0, present ? "Successfully" : "Invitation code not found");
    }
    @PostMapping(value = "/edit/slogan")
    public Result editSlogan(@RequestBody TbCfUserInfo tbCfUserInfo) {

        TbCfUserInfo user = this.user.user();

        if( user != null && tbCfUserInfo != null){
            user.setSlogan(tbCfUserInfo.getSlogan());
            repository.save(user);
        }

        return new Result<>(user);
    }
}
